/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.File;

import quicktime.*;
import quicktime.qd.*;
import quicktime.io.*;
import quicktime.app.image.*;
import quicktime.app.display.*;
import quicktime.app.QTFactory;
import quicktime.util.*;

import quicktime.app.anim.*;
import quicktime.app.actions.*;
import quicktime.std.image.*;

// For this example, we create a window with two separate canvases and a button with the label "Make Sprites jump"
// If this button is pressed, the sprites jump from one canvas to the second. The sprites consist of three spaceships-
// two rotatating and one stationary. 


public final class SpriteDemoApp extends Frame {
	public static void main (String[] args) {
		try {
			QTSession.open();							// Initialize QT native library
			SpriteDemoApp app = new SpriteDemoApp ("QT in Java");
			app.show();
			app.toFront();
		} catch (Exception e) {
			QTSession.close();
			e.printStackTrace();
		}
	}
	
	MovingQTCanvas leftCanvas;
	MovingQTCanvas rightCanvas;
	boolean c1Active;
	
	SpriteDemoApp (String s) throws Exception {
		super(s);
		setResizable (false);
		setLayout(new BorderLayout());
		setBounds (0, 0, 400, 220);
		addWindowListener (new WindowAdapter() {
			public void windowClosing (WindowEvent we) {
				leftCanvas.cleanup();
				rightCanvas.cleanup();
				QTSession.close();
				dispose();
			}
			public void windowClosed (WindowEvent we) {
				System.exit(0);
			}
		});
					
		Compositor shipAnimation = new Compositor (new QDGraphics (new QDRect(200, 200)), // create new compositor with a yellow background
													QDColor.yellow, 20, 1); 
		addSprites (shipAnimation);											// add ship sprites to the compositor
		shipAnimation.getTimer().setRate(1);								// start the compositor
			
		leftCanvas = new MovingQTCanvas (shipAnimation);					// create a new canvas and add it to the container
		add("West", leftCanvas);							
		leftCanvas.setClient (shipAnimation, true);							// make the left canvas the active canvas
		
		rightCanvas = new MovingQTCanvas (shipAnimation);					// create a second canvas and add it to the left side of the container
		add("East", rightCanvas);

		Button btn = new Button ("Click Here To Make Sprites Jump");
		btn.addActionListener (new ActionListener () {
			boolean canvas1Active = true;
			
			public void actionPerformed (ActionEvent event) {				// hook up the button and add it to the top of the window
				if (canvas1Active) {
					leftCanvas.removeClient();
					rightCanvas.doSetClient();
				} else {
					rightCanvas.removeClient();
					leftCanvas.doSetClient();
				}
				canvas1Active = ! canvas1Active;
			}
		});
		add("North", btn);
	}

	// this is used to change the rates when the clients are changed
	static int r = 2;
	
	class MovingQTCanvas extends QTCanvas {
		
		MovingQTCanvas (Compositor anim) {
			client = anim;
		}
		
		Compositor client;
		
		void doSetClient() {
			try {
				setClient (client, true);
				if (r++ % 3 == 0)
					client.getTimer().setRate(0);
				else {
					float rt = client.getTimer().getRate();
					if (rt == 0)
						client.getTimer().setRate (1);
					else
						client.getTimer().setRate (-client.getTimer().getRate());
				}
			} catch (QTException e) {
				e.printStackTrace();
			}
		}
		
		// perform application-specific cleandup and remove references to QTClient so garbage 
		// collector can reclaim memory
		public void cleanup () {
			client = null;
			removeClient();
		}
	}

	static void addSprites (Compositor comp) throws IOException, QTException {
		File matchFile = QTFactory.findAbsolutePath ("images/Ship01.pct");	//this file must exist in the directory!!!	
		ImageDataSequence dataSequence = ImageUtil.createSequence (matchFile);
		ImageDataSequence imageDataSeq = ImageUtil.makeTransparent (dataSequence, QDColor.blue);

// Build Sprites
		Matrix matrix1 = new Matrix();
		matrix1.setTx(20);
		matrix1.setTy(20);
		matrix1.setSx(0.8F);
		matrix1.setSy(0.8F);
		TwoDSprite twoDSprite1 = new TwoDSprite(imageDataSeq, 4, matrix1, true, 1);
		comp.addMember (twoDSprite1);
		
		Matrix matrix2 = new Matrix();	
		matrix2.setTx(4);
		matrix2.setTy(4);
		TwoDSprite twoDSprite2 = new TwoDSprite(imageDataSeq, 1, matrix2, true, 10);
		comp.addMember (twoDSprite2);
		
		// This needs to be a 32bit QDGraphics so the blend mode will 
		// be applied correctly to this sprite
		File shipFile = QTFactory.findAbsolutePath ("images/Ship10.pct");
		GraphicsImporterDrawer gid = new GraphicsImporterDrawer (new QTFile(shipFile));
		QDRect r = new QDRect (gid.getDescription().getWidth(), gid.getDescription().getHeight());
		ImageSpec imageSpec = ImageUtil.makeTransparent (gid, QDColor.blue, new QDGraphics (QDGraphics.kDefaultPixelFormat, r));
		Matrix matrix3 = new Matrix();	
		matrix3.setTx(50);
		matrix3.setTy(50);
		TwoDSprite twoDSprite3 = new TwoDSprite(imageSpec, matrix3, true, 8, new GraphicsMode (QDConstants.blend, QDColor.green));
		comp.addMember(twoDSprite3);

// Build ActionList (bounce action, next image action, and make the image sequence loop)
		SimpleActionList actionList = new SimpleActionList();
		ImageSequencer imageSequencer = new ImageSequencer (imageDataSeq);
		imageSequencer.setLooping (ImageSequencer.kLoopForwards);
		actionList.addMember (new NextImageAction (10, 1, imageSequencer, twoDSprite1));
		actionList.addMember (new BounceAction (5, 1, comp, twoDSprite1, 3, 2));
		ImageSequencer imageSequencer2 = new ImageSequencer (imageDataSeq);
		imageSequencer2.setLooping (ImageSequencer.kLoopForwards);
		actionList.addMember (new NextImageAction (20, 1, imageSequencer2, twoDSprite2));
		actionList.addMember (new BounceAction (20, 1, comp, twoDSprite2, 4, 3));
		comp.addController(actionList);
	}
}
